package fishrungames.yelpmapapp

import android.support.v4.app.Fragment
import android.os.Bundle
import android.support.design.widget.AppBarLayout

import android.support.v7.widget.LinearLayoutManager

import android.view.ViewGroup
import android.view.LayoutInflater
import android.view.View
import fishrungames.yelpmapapp.records.BusinessDetailRecord
import fishrungames.yelpmapapp.records.ReviewListRecord
import android.support.v7.widget.RecyclerView

import android.widget.ImageView
import android.widget.TextView
import com.koushikdutta.ion.Ion
import fishrungames.yelpmapapp.records.MapMarkerRecord



/**
 * Created by mephi on 10.06.2017.
 */

class DetailFragment : Fragment()
{

    var detailRecyclerAdapter : DetailRecyclerAdapter? = null

    var mapMarkerRecord : MapMarkerRecord? = null

    var backdropImageView : ImageView? = null

    var backgroundTitleTextView : TextView? = null
    var toolbarTitleTextView : TextView? = null

    var detailCoordinatorChangeOffset : Int = 0
    var detailCoordinatorOffset : Int = 0

    override fun onCreateView(inflater: LayoutInflater?, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        detailCoordinatorChangeOffset = context.resources.getInteger(R.integer.detailCoordinatorChangeOffset)
        detailCoordinatorOffset = context.resources.getInteger(R.integer.detailCoordinatorOffset)

        val rootView = inflater!!.inflate(R.layout.detail_fragment, container, false)

        val appBarLayout = rootView.findViewById(R.id.appBarLayout) as AppBarLayout

        appBarLayout.addOnOffsetChangedListener {
            _, verticalOffset ->
            run {
                if (-verticalOffset > detailCoordinatorChangeOffset) {
                    val alpha = (-verticalOffset - detailCoordinatorChangeOffset).toFloat() / (detailCoordinatorOffset - detailCoordinatorChangeOffset).toFloat()
                    toolbarTitleTextView?.alpha = alpha
                    backgroundTitleTextView?.alpha = 1F - alpha
                }
                else
                {
                    toolbarTitleTextView?.alpha = 0F
                    backgroundTitleTextView?.alpha = 1F
                }
            }
        }

        val detailRecyclerView = rootView.findViewById(R.id.detailRecyclerView) as RecyclerView

        backdropImageView = rootView.findViewById(R.id.backdropImageView) as ImageView

        backgroundTitleTextView = rootView.findViewById(R.id.backgroundTitleTextView) as TextView
        toolbarTitleTextView = rootView.findViewById(R.id.toolbarTitleTextView) as TextView

        detailRecyclerView.layoutManager = LinearLayoutManager(this.context)

        mapMarkerRecord = arguments?.getSerializable("mapMarkerRecord") as? MapMarkerRecord

        if (mapMarkerRecord != null && mapMarkerRecord!!.id != null) {

            Ion.with(backdropImageView)
                    .placeholder(R.drawable.placeholder)
                    .error(R.drawable.placeholder)
                    .fadeIn(true)
                    .load(mapMarkerRecord!!.image_url)

            toolbarTitleTextView?.text = mapMarkerRecord!!.name
            backgroundTitleTextView?.text = mapMarkerRecord!!.name

            detailRecyclerAdapter = DetailRecyclerAdapter(context, mapMarkerRecord!!)

            detailRecyclerView.adapter = detailRecyclerAdapter

            (activity as? YelpMapActivity)?.dataProvider?.requestBusinessDetail(mapMarkerRecord!!.id!!)
            (activity as? YelpMapActivity)?.dataProvider?.requestReviewList(mapMarkerRecord!!.id!!)

        }

        return rootView
    }

    fun updateBusinessDetail(businessDetailRecord : BusinessDetailRecord)
    {
        if (businessDetailRecord.id != null && businessDetailRecord.id == mapMarkerRecord?.id) {

            Ion.with(backdropImageView)
                    .error(R.drawable.placeholder)
                    .fadeIn(true)
                    .load(businessDetailRecord.image_url)

            toolbarTitleTextView?.text = businessDetailRecord.name
            backgroundTitleTextView?.text = mapMarkerRecord!!.name

            detailRecyclerAdapter?.businessDetailRecord = businessDetailRecord

            detailRecyclerAdapter?.notifyDataSetChanged()
        }
    }

    fun  updateReviewList(reviewListRecordPair : Pair<String, ReviewListRecord>)
    {
        if (reviewListRecordPair.first == mapMarkerRecord?.id) {

            detailRecyclerAdapter?.reviewListRecordPair = reviewListRecordPair

            detailRecyclerAdapter?.notifyDataSetChanged()
        }
    }
}