package fishrungames.yelpmapapp

import android.content.Context
import android.support.v4.content.ContextCompat
import android.support.v7.widget.RecyclerView
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import android.view.LayoutInflater
import android.view.View.GONE
import android.view.View.VISIBLE
import android.widget.ImageView
import android.widget.ProgressBar
import com.koushikdutta.ion.Ion
import fishrungames.yelpmapapp.records.BusinessDetailRecord
import fishrungames.yelpmapapp.records.MapMarkerRecord
import fishrungames.yelpmapapp.records.ReviewListRecord
import java.text.DecimalFormat


/**
 * Created by mephi on 10.06.2017.
 */


class DetailRecyclerAdapter(val context : Context, val mapMarkerRecord : MapMarkerRecord) : RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    var businessDetailRecord : BusinessDetailRecord? = null

    var reviewListRecordPair : Pair<String, ReviewListRecord>? = null

    internal class PhoneViewHolder(v: View) : RecyclerView.ViewHolder(v) {

        var phoneTextView: TextView = v.findViewById(R.id.phoneTextView) as TextView

    }

    internal class AddressViewHolder(v: View) : RecyclerView.ViewHolder(v) {

        var addressTextView: TextView = v.findViewById(R.id.addressTextView) as TextView

    }

    internal class ReviewTitleViewHolder(v: View) : RecyclerView.ViewHolder(v) {

        var noReviewsTextView: TextView = v.findViewById(R.id.noReviewsTextView) as TextView

        var progressBar: ProgressBar = v.findViewById(R.id.progressBar) as ProgressBar



    }

    internal class RatingViewHolder(v: View) : RecyclerView.ViewHolder(v) {

        var ratingTextView : TextView = v.findViewById(R.id.ratingTextView) as TextView

        val starImageView1 = v.findViewById(R.id.starImageView1) as ImageView
        val starImageView2 = v.findViewById(R.id.starImageView2) as ImageView
        val starImageView3 = v.findViewById(R.id.starImageView3) as ImageView
        val starImageView4 = v.findViewById(R.id.starImageView4) as ImageView
        val starImageView5 = v.findViewById(R.id.starImageView5) as ImageView

    }

    internal class ReviewViewHolder(v: View) : RecyclerView.ViewHolder(v) {

        var nameTextView : TextView = v.findViewById(R.id.nameTextView) as TextView

        var messageTextView : TextView = v.findViewById(R.id.messageTextView) as TextView

        var dateTextView : TextView = v.findViewById(R.id.dateTextView) as TextView

        var ratingTextView : TextView = v.findViewById(R.id.ratingTextView) as TextView

        val starImageView1 = v.findViewById(R.id.starImageView1) as ImageView
        val starImageView2 = v.findViewById(R.id.starImageView2) as ImageView
        val starImageView3 = v.findViewById(R.id.starImageView3) as ImageView
        val starImageView4 = v.findViewById(R.id.starImageView4) as ImageView
        val starImageView5 = v.findViewById(R.id.starImageView5) as ImageView

        val profilePictureImageView = v.findViewById(R.id.profilePictureImageView) as ImageView



    }



    override fun getItemViewType(position: Int): Int {
        return position
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        when (viewType) {
            0 -> {
                val v = LayoutInflater.from(parent.context).inflate(R.layout.recycler_rating, parent, false)
                return RatingViewHolder(v)
            }
            1 -> {
                val v = LayoutInflater.from(parent.context).inflate(R.layout.recycler_phone, parent, false)
                return PhoneViewHolder(v)
            }
            2 -> {
                val v = LayoutInflater.from(parent.context).inflate(R.layout.recycler_address, parent, false)
                return AddressViewHolder(v)
            }
            3 -> {
                val v = LayoutInflater.from(parent.context).inflate(R.layout.recycler_review_title, parent, false)
                return ReviewTitleViewHolder(v)
            }
            else -> {
                val v = LayoutInflater.from(parent.context).inflate(R.layout.recycler_review, parent, false)
                return ReviewViewHolder(v)
            }
        }
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        when (holder.itemViewType) {
            0 -> {
                val ratingViewHolder = holder as RatingViewHolder

                val rating = businessDetailRecord?.rating ?: (mapMarkerRecord.rating ?: 0.0)

                val formatter = DecimalFormat("#0.00")

                ratingViewHolder.ratingTextView.text = formatter.format(rating)

                ratingViewHolder.starImageView1.setImageDrawable(ContextCompat.getDrawable(context, Utils.getStarRatingId(rating, 0.0)))
                ratingViewHolder.starImageView2.setImageDrawable(ContextCompat.getDrawable(context, Utils.getStarRatingId(rating, 1.0)))
                ratingViewHolder.starImageView3.setImageDrawable(ContextCompat.getDrawable(context, Utils.getStarRatingId(rating, 2.0)))
                ratingViewHolder.starImageView4.setImageDrawable(ContextCompat.getDrawable(context, Utils.getStarRatingId(rating, 3.0)))
                ratingViewHolder.starImageView5.setImageDrawable(ContextCompat.getDrawable(context, Utils.getStarRatingId(rating, 4.0)))

            }
            1 -> {
                val phoneViewHolder = holder as PhoneViewHolder

                if (businessDetailRecord?.display_phone != null) {

                    phoneViewHolder.phoneTextView.text = businessDetailRecord?.display_phone
                }
                else
                {
                    phoneViewHolder.phoneTextView.text = mapMarkerRecord.display_phone
                }
            }

            2 -> {
                val addressViewHolder = holder as AddressViewHolder

                if (businessDetailRecord?.location?.display_address != null) {

                    addressViewHolder.addressTextView.text = businessDetailRecord?.location?.display_address?.joinToString(" ")
                }
                else
                {
                    addressViewHolder.addressTextView.text = mapMarkerRecord.location?.display_address?.joinToString(" ")
                }

            }

            3 -> {
                val reviewTitleViewHolder = holder as ReviewTitleViewHolder

                if (reviewListRecordPair == null)
                {
                    reviewTitleViewHolder.progressBar.visibility = VISIBLE
                    reviewTitleViewHolder.noReviewsTextView.visibility = GONE
                }
                else
                {
                    reviewTitleViewHolder.progressBar.visibility = GONE
                    reviewTitleViewHolder.noReviewsTextView.visibility = if (getReviewCount() > 0) GONE else VISIBLE
                }

            }

            else -> {
                val reviewViewHolder = holder as ReviewViewHolder

                val reviewRecord = reviewListRecordPair?.second?.reviews?.get(position - 4)

                if (reviewRecord != null) {

                    reviewViewHolder.nameTextView.text = reviewRecord.user?.name

                    reviewViewHolder.messageTextView.text = reviewRecord.text

                    reviewViewHolder.dateTextView.text = reviewRecord.time_created

                    val rating = reviewRecord.rating ?: 0.0
                    val formatter = DecimalFormat("#0.00")

                    reviewViewHolder.ratingTextView.text = formatter.format(rating)

                    reviewViewHolder.starImageView1.setImageDrawable(ContextCompat.getDrawable(context, Utils.getStarRatingId(rating, 0.0)))
                    reviewViewHolder.starImageView2.setImageDrawable(ContextCompat.getDrawable(context, Utils.getStarRatingId(rating, 1.0)))
                    reviewViewHolder.starImageView3.setImageDrawable(ContextCompat.getDrawable(context, Utils.getStarRatingId(rating, 2.0)))
                    reviewViewHolder.starImageView4.setImageDrawable(ContextCompat.getDrawable(context, Utils.getStarRatingId(rating, 3.0)))
                    reviewViewHolder.starImageView5.setImageDrawable(ContextCompat.getDrawable(context, Utils.getStarRatingId(rating, 4.0)))

                    Ion.with(reviewViewHolder.profilePictureImageView)
                            .placeholder(R.drawable.placeholder)
                            .error(R.drawable.placeholder)
                            .fadeIn(true)
                            .load(reviewRecord.user?.image_url)
                }
            }
        }
    }

    override fun getItemCount(): Int {

        return 4 + getReviewCount()
    }

    private fun getReviewCount() : Int {
        return reviewListRecordPair?.second?.reviews?.count() ?: 0
    }
}